/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the "License").  You may not use this file except
 * in compliance with the License.
 *
 * You can obtain a copy of the license at
 * glassfish/bootstrap/legal/CDDLv1.0.txt or
 * https://glassfish.dev.java.net/public/CDDLv1.0.html.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * HEADER in each file and include the License file at
 * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
 * add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your
 * own identifying information: Portions Copyright [yyyy]
 * [name of copyright owner]
 */

package com.sun.grizzly.jruby;

import com.sun.grizzly.arp.AsyncProcessorTask;
import com.sun.grizzly.http.AsyncExecutor;
import com.sun.grizzly.http.AsyncFilter;
import com.sun.grizzly.http.AsyncHandler;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * Simple implementation to demonstrate the asynchronous request processing.
 * The request will be parsed and the Servlet.service() will be invoked using
 * the Thread that execute this filter. Then 10 seconds after, another thread
 * (from ScheduledThreadPoolExecutor) will finish the response.
 *
 * This demonstrate that you can control when a Servlet.service() is invoked
 * and by which thread.
 *
 * @author Jeanfrancois Arcand
 */
public class DelayedAsyncFilter implements AsyncFilter, Callable {

    
    private ScheduledThreadPoolExecutor scheduler
            = new ScheduledThreadPoolExecutor(1);
    
    
    private ConcurrentLinkedQueue<AsyncProcessorTask> delayedQueue
            = new ConcurrentLinkedQueue<AsyncProcessorTask>();
    
    
    public boolean doFilter(AsyncExecutor asyncExecutor) {
        delayedQueue.offer((AsyncProcessorTask)asyncExecutor.getAsyncTask());
        // Let the current thread continue
        System.out.println("\nParking the request for 10 seconds");
        scheduler.schedule(this,10,TimeUnit.SECONDS);
        return false;
    }
    
    
    /**
     * This method will be invoked the ScheduledThreadPoolExecutor own thread
     * and will flush the response to the client.
     */
    public Object call() throws Exception{
        System.out.println("Resuming the request");
        
        AsyncProcessorTask asyncProcessorTask = delayedQueue.poll();
        AsyncHandler asyncHandler = asyncProcessorTask.getAsyncExecutor()
            .getAsyncHandler();
        asyncProcessorTask.getProcessorTask().invokeAdapter();        
        asyncHandler.handle(asyncProcessorTask);
        return null;
    }
}
