/*
 * The contents of this file are subject to the terms 
 * of the Common Development and Distribution License 
 * (the License).  You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at 
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing 
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL 
 * Header Notice in each file and include the License file 
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.  
 * If applicable, add the following below the CDDL Header, 
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information: 
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 */
package com.sun.grizzly.jruby;

import java.io.IOException;
import java.util.logging.Level;
import com.sun.grizzly.http.SelectorThread;
import com.sun.grizzly.standalone.StaticStreamAlgorithm;


/**
 * JRuby on rails implementation of Grizzly SelectorThread
 * @author TAKAI Naoto
 * @author Jeanfrancois Arcand
 */
public class RailsSelectorThread extends SelectorThread {

    private String jrubyLib = null;

    private int numberOfRuntime = 5;

    private RubyObjectPool pool = null;

    private String railsRoot = null;

    public String getRailsRoot() {
        return railsRoot;
    }

    
    @Override
    public void initEndpoint() throws IOException, InstantiationException {
        setupSystemProperties();
        initializeRubyRuntime();

        this.adapter = new RailsAdapter(pool);
        
        setBufferResponse(false);
        algorithmClassName = StaticStreamAlgorithm.class.getName();
        super.initEndpoint();
    }

    
    public void setNumberOfRuntime(int numberOfRuntime) {
        this.numberOfRuntime = numberOfRuntime;
    }

    
    public void setRailsRoot(String railsRoot) {
        this.railsRoot = railsRoot;
    }

    
    @Override
    public synchronized void stopEndpoint() {
        pool.stop();

        super.stopEndpoint();
    }

    
    protected void initializeRubyRuntime() {
        pool = new RubyObjectPool();
        pool.setNumberOfRuntime(numberOfRuntime);
        pool.setJrubyLib(jrubyLib);
        pool.setRailsRoot(railsRoot);

        try {
            pool.start();
        } catch (Throwable t) {
            t.printStackTrace();
            logger.log(Level.WARNING, t.getMessage());
        }
    }

    
    protected void setupSystemProperties() {
        jrubyLib = System.getProperty("jruby.lib");
        if (jrubyLib == null) {
            jrubyLib = System.getenv().get("JRUBY_HOME") + "/lib";
            System.setProperty("jruby.lib", jrubyLib);
        }
    }
}
